//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Canvas/MaskEditorCanvas.cpp
//! @brief     Implements class MaskEditorCanvas.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Canvas/MaskEditorCanvas.h"
#include "Device/Data/Datafield.h"
#include "GUI/Model/Data/Data2DItem.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/Model/Mask/MaskList.h"
#include "GUI/Model/Project/ProjectDocument.h"
#include "GUI/View/PlotScale/SavePlotAssistant.h"
#include "GUI/View/Plotter/ColorMap.h"
#include "GUI/View/Plotter/PlotStatusLabel.h"
#include "GUI/View/Scene/MaskGraphicsScene.h"
#include "GUI/View/Scene/MaskGraphicsView.h"
#include <QVBoxLayout>

MaskEditorCanvas::MaskEditorCanvas()
    : m_scene(new MaskGraphicsScene)
    , m_view(new MaskGraphicsView(m_scene))
    , m_status_label(new PlotStatusLabel(nullptr, this))
    , m_data_item(nullptr)
{
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    auto* layout = new QVBoxLayout;
    setLayout(layout);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    layout->addWidget(m_view);
    layout->addWidget(m_status_label);

    connect(m_view, &MaskGraphicsView::changeActivityRequest, this,
            &MaskEditorCanvas::changeActivityRequest);
    connect(m_view, &MaskGraphicsView::deleteSelectedRequest, this,
            &MaskEditorCanvas::deleteSelectedRequest);
}

MaskEditorCanvas::~MaskEditorCanvas() = default;

void MaskEditorCanvas::updateCanvas(Data2DItem* data_item)
{
    m_data_item = data_item;

    m_scene->associateItems(data_item, data_item->getOrCreateMaskModel());

    m_scene->updateSize(m_view->size());

    m_status_label->setPlot(m_scene->colorMap());
}

void MaskEditorCanvas::resetCanvas()
{
    m_data_item = nullptr;
    m_scene->dissociateItems();
    m_status_label->reset();
}

void MaskEditorCanvas::onPresentationChange(bool pixelized)
{
    m_scene->clearSelection(); // important to avoid crash (unsubscribe while calling subscribers)

    ASSERT(m_data_item);

    if (pixelized) {
        if (Datafield* maskedData = m_data_item->createMaskedField()) {
            // store data backup
            m_backup_data.reset(m_data_item->c_field()->clone());
            m_backup_interpolated = m_data_item->isInterpolated();

            m_data_item->setDatafield(*maskedData);
            m_data_item->setInterpolated(false);
        } else {
            m_backup_data.reset();
        }

    } else {
        if (m_backup_data) {
            m_data_item->setDatafield(*m_backup_data);
            m_data_item->setInterpolated(m_backup_interpolated);
        }
    }

    if (auto* container = m_data_item->maskContainerItem())
        for (MaskItem* mask : container->modifiableMaskItems())
            mask->setIsVisible(!pixelized && mask->wasVisible());
}

//! Saves plot into project directory.

void MaskEditorCanvas::onSavePlotRequest()
{
    ASSERT(gDoc);
    QString dirname = gDoc->userExportDir();
    GUI::Plot::savePlot(dirname, m_scene->colorMap()->customPlot(), m_data_item->c_field());
}

void MaskEditorCanvas::onResetViewRequest()
{
    m_view->onResetViewRequest();

    if (m_data_item->isZoomed())
        m_data_item->resetView();
    else
        setZoomToROI();
    gDoc->setModified();
}

void MaskEditorCanvas::setZoomToROI()
{
    if (MaskList* item = m_data_item->maskContainerItem()) {
        if (auto* roiItem = item->regionOfInterestItem()) {
            m_data_item->setXrange(roiItem->xLow(), roiItem->xUp());
            m_data_item->setYrange(roiItem->yLow(), roiItem->yUp());
        }
    }
}
