//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Canvas/ProjectedGraphsCanvas.cpp
//! @brief     Implements class ProjectedGraphsCanvas.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Canvas/ProjectedGraphsCanvas.h"
#include "GUI/Model/Data/Data2DItem.h"
#include "GUI/View/Plotter/ProjectionsPlot.h"
#include <QVBoxLayout>

namespace {

const int horizontal_projection_tab = 0;
const int vertical_projection_tab = 1;

} // namespace

ProjectedGraphsCanvas::ProjectedGraphsCanvas(QWidget* parent)
    : QWidget(parent)
    , m_xProjection(new ProjectionsPlot(Qt::Horizontal))
    , m_yProjection(new ProjectionsPlot(Qt::Vertical))
    , m_tabWidget(new QTabWidget)
{
    auto* layout = new QVBoxLayout(this);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);

    m_tabWidget->setTabPosition(QTabWidget::North);
    m_tabWidget->insertTab(Qt::Horizontal, m_xProjection, "Horizontal");
    m_tabWidget->insertTab(Qt::Vertical, m_yProjection, "Vertical");

    layout->addWidget(m_tabWidget);

    setConnected(true);
}

void ProjectedGraphsCanvas::setData2DItem(Data2DItem* data_item)
{
    m_xProjection->setData2DItem(data_item);
    m_yProjection->setData2DItem(data_item);
}

void ProjectedGraphsCanvas::disconnectItem()
{
    m_xProjection->disconnectItems();
    m_yProjection->disconnectItems();
}

void ProjectedGraphsCanvas::onActivityChanged(MaskFlags::MaskMode mask_mode)
{
    setConnected(false);

    if (mask_mode == MaskFlags::HORIZONTAL_LINE_MODE)
        m_tabWidget->setCurrentIndex(horizontal_projection_tab);
    else if (mask_mode == MaskFlags::VERTICAL_LINE_MODE)
        m_tabWidget->setCurrentIndex(vertical_projection_tab);

    setConnected(true);
}

void ProjectedGraphsCanvas::onMarginsChanged(double left, double right)
{
    m_xProjection->onMarginsChanged(left, right);
    m_yProjection->onMarginsChanged(left, right);
}

void ProjectedGraphsCanvas::onTabChanged(int tab_index)
{
    if (tab_index == horizontal_projection_tab)
        emit changeActivityRequest(MaskFlags::HORIZONTAL_LINE_MODE);
    else if (tab_index == vertical_projection_tab)
        emit changeActivityRequest(MaskFlags::VERTICAL_LINE_MODE);
}

void ProjectedGraphsCanvas::setConnected(bool isConnected)
{
    if (isConnected)
        connect(m_tabWidget, &QTabWidget::currentChanged, this,
                &ProjectedGraphsCanvas::onTabChanged);
    else
        disconnect(m_tabWidget, &QTabWidget::currentChanged, this,
                   &ProjectedGraphsCanvas::onTabChanged);
}
