//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Canvas/ProjectionsEditorCanvas.cpp
//! @brief     Implements class ProjectionsEditorCanvas.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Canvas/ProjectionsEditorCanvas.h"
#include "GUI/Model/Data/Data2DItem.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/Model/Mask/MaskeditorListmodel.h"
#include "GUI/Model/Mask/ProjectionList.h"
#include "GUI/Model/Project/ProjectDocument.h"
#include "GUI/View/Plotter/ColorMap.h"
#include "GUI/View/Plotter/PlotEventHelper.h"
#include "GUI/View/Plotter/PlotStatusLabel.h"
#include "GUI/View/Scene/MaskGraphicsScene.h"
#include "GUI/View/Scene/MaskGraphicsView.h"
#include <QItemSelectionModel>
#include <QVBoxLayout>

ProjectionsEditorCanvas::ProjectionsEditorCanvas()
    : m_scene(new MaskGraphicsScene)
    , m_view(new MaskGraphicsView(m_scene))
    , m_status_label(new PlotStatusLabel(nullptr, this))
    , m_current_activity(MaskFlags::HORIZONTAL_LINE_MODE)
{
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    auto* layout = new QVBoxLayout(this);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    layout->addWidget(m_view);
    layout->addWidget(m_status_label);

    connect(m_view, &MaskGraphicsView::changeActivityRequest, this,
            &ProjectionsEditorCanvas::changeActivityRequest);
    connect(m_view, &MaskGraphicsView::deleteSelectedRequest, this,
            &ProjectionsEditorCanvas::deleteSelectedRequest);

    // automatically switch to the appropriate projection tab
    connect(m_scene, &MaskGraphicsScene::lineItemMoved, [this](LineItem* sender) {
        if (dynamic_cast<HorizontalLineItem*>(sender)
            && (m_current_activity != MaskFlags::VERTICAL_LINE_MODE))
            emit changeProjectionsTabRequest(MaskFlags::HORIZONTAL_LINE_MODE);
        if (dynamic_cast<VerticalLineItem*>(sender)
            && (m_current_activity != MaskFlags::HORIZONTAL_LINE_MODE))
            emit changeProjectionsTabRequest(MaskFlags::VERTICAL_LINE_MODE);
    });
}

ProjectionsEditorCanvas::~ProjectionsEditorCanvas() = default;

void ProjectionsEditorCanvas::updateCanvas(Data2DItem* data_item)
{
    MaskeditorListmodel* mask_list_model = data_item->getOrCreateProjectionModel();
    ASSERT(mask_list_model);

    m_scene->associateItems(data_item, mask_list_model);

    // notify ProjectionPlot about the changes
    connect(m_scene, &MaskGraphicsScene::lineItemProcessed, data_item,
            &Data2DItem::projectionCreated, Qt::UniqueConnection);
    connect(m_scene, &MaskGraphicsScene::lineItemMoved, data_item,
            &Data2DItem::projectionPositionChanged, Qt::UniqueConnection);
    connect(m_scene, &MaskGraphicsScene::lineItemDeleted, data_item, &Data2DItem::projectionGone,
            Qt::UniqueConnection);

    m_scene->updateSize(m_view->size());

    m_data_item = data_item;

    auto* cm = m_scene->colorMap();
    ASSERT(cm);
    connect(cm->eventHelper(), &PlotEventHelper::enteringPlot, this,
            &ProjectionsEditorCanvas::onEnteringColorMap, Qt::UniqueConnection);
    connect(cm->eventHelper(), &PlotEventHelper::leavingPlot, this,
            &ProjectionsEditorCanvas::onLeavingColorMap, Qt::UniqueConnection);
    connect(cm->eventHelper(), &PlotEventHelper::positionChanged, this,
            &ProjectionsEditorCanvas::onPositionChanged, Qt::UniqueConnection);
    connect(cm, &ColorMap::marginsChanged, this, &ProjectionsEditorCanvas::marginsChanged,
            Qt::UniqueConnection);

    m_status_label->setPlot(cm);

    onLeavingColorMap();
    m_scene->onActivityChanged(m_current_activity);
}

void ProjectionsEditorCanvas::resetCanvas()
{
    auto* cm = m_scene->colorMap();
    ASSERT(cm);
    disconnect(cm->eventHelper(), &PlotEventHelper::enteringPlot, this,
               &ProjectionsEditorCanvas::onEnteringColorMap);
    disconnect(cm->eventHelper(), &PlotEventHelper::leavingPlot, this,
               &ProjectionsEditorCanvas::onLeavingColorMap);
    disconnect(cm->eventHelper(), &PlotEventHelper::positionChanged, this,
               &ProjectionsEditorCanvas::onPositionChanged);
    disconnect(cm, &ColorMap::marginsChanged, this, &ProjectionsEditorCanvas::marginsChanged);

    m_data_item = nullptr;
    m_scene->dissociateItems();
}

void ProjectionsEditorCanvas::onEnteringColorMap()
{
    if (m_live_projection)
        return;

    if (m_current_activity == MaskFlags::HORIZONTAL_LINE_MODE)
        m_live_projection = std::make_unique<HorizontalLineItem>();
    else if (m_current_activity == MaskFlags::VERTICAL_LINE_MODE)
        m_live_projection = std::make_unique<VerticalLineItem>();

    if (m_live_projection) {
        m_live_projection->setIsVisible(false);
        emit m_data_item->projectionPositionChanged(m_live_projection.get());
    }
}

void ProjectionsEditorCanvas::onLeavingColorMap()
{
    if (m_live_projection) {
        disconnect(m_live_projection.get(), nullptr, m_data_item, nullptr);
        emit m_data_item->projectionGone(m_live_projection.get());
        m_live_projection.reset();
    }
}

void ProjectionsEditorCanvas::onPositionChanged(double x, double y)
{
    if (m_live_projection) {
        if (m_current_activity == MaskFlags::HORIZONTAL_LINE_MODE)
            dynamic_cast<HorizontalLineItem*>(m_live_projection.get())->setPos(y);
        else if (m_current_activity == MaskFlags::VERTICAL_LINE_MODE)
            dynamic_cast<VerticalLineItem*>(m_live_projection.get())->setPos(x);
    }
}

void ProjectionsEditorCanvas::onResetViewRequest()
{
    m_view->onResetViewRequest();
    m_data_item->resetView();
    gDoc->setModified();
}

void ProjectionsEditorCanvas::setProjectionsCanvasMode(MaskFlags::MaskMode mask_mode)
{
    m_current_activity = mask_mode;
    m_scene->onActivityChanged(mask_mode);
    onLeavingColorMap();
}
